/*
** Copyright 1992 by Markku Savela and
**	Technical Research Centre of Finland
**
** The author maybe contacted via:
**
**    Mail:	Markku Savela
**		Technical Research Centre of Finland
**		Telecommunications Laboratory
**		Otakaari 7 B, P.O. Box 34
**		SF-02151 ESPOO
**		Finland
**
**    Phone:	+358 0 456 6509
**    Email:	savela@tel.vtt.fi
*/
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xew/XewInit.h>
#include <X11/Xew/VideoP.h>
#ifdef H261_TIK
#	include <h261/h261lib.h>
#endif
/* #include <mpeg/mpeglib.h> */

#define offset(field) XtOffsetOf(XeVideoRec, video.field)

static XtResource resources[] =
    {
	{XtNcontrast, XtCContrast, XtRInt, sizeof(int),
		 offset(contrast), XtRImmediate, 0},
	{XtNbrightness, XtCBrightness, XtRInt, sizeof(int),
		 offset(brightness), XtRImmediate, 0},
        {XtNuseShm, XtCuseShm, XtRBoolean, sizeof(Boolean),
                 offset(useShm), XtRImmediate, (XtPointer)0},
    };
#undef offset

static void ClassPartInitialize(), ClassInitialize(), Realize();
static void Initialize(), Redisplay(), Destroy();
static Boolean SetValues();

#ifdef H261_TIK
/* Module static variables */
static int videoOn = FALSE;
static int First = TRUE;
static int videoId = -1;
#endif

XeVideoClassRec xeVideoClassRec = {
  { /* core fields */
    /* superclass		*/	(WidgetClass) &xeAnimationClassRec,
    /* class_name		*/	"XeVideo",
    /* widget_size		*/	sizeof(XeVideoRec),
    /* class_initialize		*/	ClassInitialize,
    /* class_part_initialize	*/	ClassPartInitialize,
    /* class_inited		*/	FALSE,
    /* initialize		*/	Initialize,
    /* initialize_hook		*/	NULL,
    /* realize			*/	XtInheritRealize,
    /* actions			*/	NULL,
    /* num_actions		*/	0,
    /* resources		*/	resources,
    /* num_resources		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	XtExposeCompressMultiple,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	Destroy,
    /* resize			*/	NULL,
    /* expose			*/	Redisplay,
    /* set_values		*/	SetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	NULL,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
  },
  { /* XeBasic fields */
    /* not used			*/	0
  },
  { /* XeAnimation fields */
    /* not used			*/	0
  },
  { /* XeVideo fields */
    /* not used			*/	0
  }
};

WidgetClass xeVideoWidgetClass = (WidgetClass)&xeVideoClassRec;

static void ClassInitialize()
    {
        printf("Video widget ClassInitialize\n");
	XewInitializeWidgetSet();
    }

static void ClassPartInitialize(class)
WidgetClass class;
    {
        printf("Video widget ClassPartInitialize\n");
    }

static void Initialize(request, new, args, num_args)
Widget request, new;
ArgList args;
Cardinal *num_args;
    {
#ifdef H261_TIK
        XeVideoWidget w = (XeVideoWidget)new;
        printf("Video widget Initialize\n");
	
        /* Perhaps it is possible to verify format from data */
        /* Now format must be set before processing.         */
        if (w->basic.content_format == NULL)
            {
		XeWidgetWarningMsg((Widget)w, "contentFormatUnset",
				   "Content format is not set",
				   (String *)NULL, 0);
            }
        else if (w->basic.content_file != NULL)
            {
		int winId;
		switch (w->basic.content_format)
		    {
		    case XeContentFormat_H261:
			winId = XtWindow(w);
			fprintf(stderr, "Window Id: %d\n", winId);
			videoId = initH261Video(w->basic.content_file,0,0,1,1);
			if (videoId == -1)
				XeWidgetWarningMsg
					((Widget)w, "fileNotFound",
					 "File %s not found",
					 (String *)w->basic.content_file, 1);
			else
			    {
				if (request->core.width == 0)
					new->core.width=getH261width(videoId);
				if (request->core.height == 0)
					new->core.height=getH261height(videoId);
			    }
			break;
		    case XeContentFormat_MPEG:
			if (!initMPEGVideo(w->basic.content_file))
			    {
				XeWidgetWarningMsg((Widget)w, "fileNotFound",
						   "File %s not found",
						   (String *)w->basic.content_file, 1);
			    };
			break;
		    default:
			XeWidgetWarningMsg((Widget)w, "contentFormatUnknown",
					   "Unknown video format set",
					   (String *)NULL, 0);
			break;
		    }
	    }
        else if (w->basic.content_stream != NULL)
            {
            }
        else if (w->basic.content_string != NULL)
            {
            }
        else
	    {
	    }
#endif
    }

static void Realize(w, valueMask, attributes)
Widget w;
Mask *valueMask;
XSetWindowAttributes *attributes;
    {
        printf("Video widget Realize\n");
    }

static Boolean SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
    {
	return False;   
    }

static void Redisplay(wid, event, r)
Widget wid;
XExposeEvent *event;
Region r;
    {
#if H261_TIK
        XeVideoWidget w = (XeVideoWidget)wid;
	
        printf("Video widget Expose\n");
        if (!XtIsRealized(wid))
		return;
        if (videoOn)
	    {
		printf("Send refresh to video server\n");
		refreshVideo(videoId);
	    }
        if ((w->basic.content_file != NULL) && First)
	    {
		forkH261Video(videoId, XtWindow(w));
		First = FALSE;
		noXShmVideo(videoId);
		playVideo(videoId);
		videoOn = TRUE;
	    }
#endif
    }

static void Destroy(wid)
Widget wid;
    {
#ifdef TIK_H261
        fprintf(stderr, "Destroy video widget\n");
        fflush(stderr);
        if (videoOn)
		quitVideo(videoId);
#endif
    }


